#!/usr/bin/perl
################################################################################
#
# dirsynch - synchronized specified files from source directory to
#           target directory
# by Jim Tisdall 
#
################################################################################

$USAGE = '
 Usage: dirsynch [-s] [-f filelist] [sourcedirectory] targetdirectory

        where filelist is a file containing names of files, one
        per line (pathnames relative to sourcedirectory and to
        targetdirectory) to synchronize from the sourcedirectory.
        If filelist is not specified, the file .dirsynch is read.
        if sourcedirectory is not specified, the current
        directory is assumed.
        -s directs the program to strip all directories from the
        filenames in the filelist or sourcedirectory before copying
        to the targetdirectory

';

############
# globals
############
$filelist = '';
$sourcedir = '';
$targetdir = '';
$strippaths = 0;


############
# Get command-line arguments
############

if( ! @ARGV) {
    print STDERR "$USAGE";
    exit;
}elsif( $ARGV[0] =~ /^-f/) {
    $ARGV[0] =~ /^-f(.*)/;
    if($1) {
        $filelist = $1;
        unless(-e "$filelist") {
            print STDERR "Cannot find file $filelist\n";
            print STDERR "$USAGE";
            exit;
        }
        shift(@ARGV);
    }else{
        unless(scalar(@ARGV) > 2) {
            print STDERR "$USAGE";
            exit;
        }
        $filelist = $ARGV[1];
        shift(@ARGV);
        shift(@ARGV);
    }
}elsif( ! -e '.dirsync' ) {
    print STDERR "$USAGE";
    exit;
}
if( grep(/^-s$/,$ARGV[0])) {
    $strippaths = 1;
    @ARGV = grep(!/^-s$/,@ARGV);
}


if(scalar(@ARGV) == 1) {
    $sourcedir = '.';
    $targetdir = $ARGV[0];
}elsif(scalar(@ARGV) == 2) {
    $sourcedir = $ARGV[0];
    $targetdir = $ARGV[1];
}else{
    print STDERR "$USAGE";
    exit;
}


############
# check that directories exist
############

unless(-e "$sourcedir" and -d "$sourcedir") {
    print STDERR "Cannot find sourcedir $sourcedir\n";
    print STDERR "$USAGE";
    exit;
}
unless(-e "$targetdir" and -d "$targetdir") {
    print STDERR "Cannot find targetdir $targetdir\n";
    print STDERR "$USAGE";
    exit;
}

############
# read file list
############

unless(open(FILELIST,"$filelist")) {
    print STDERR "Cannot open $filelist for reading\n";
    print STDERR "$USAGE";
    exit;
}

my(@filelist) = <FILELIST>;

# remove comments
@filelist = grep(! /^\s*#/, @filelist);
# remove blank lines
@filelist = grep(! /^\s*$/, @filelist);
# remove new lines
map(chomp, @filelist);

foreach $file (@filelist) 
{
    &mirror($file, '/usr/bin/cmp', $sourcedir, $targetdir,$strippaths);
}


################################################################################
# Subroutines
################################################################################

sub mirror {
    my($file, $cmp, $sdir, $tdir, $strippaths) = @_;

    my $cmpout;
    my $cpout;

    my $command="$cmp ${sdir}/${file} ${tdir}/${file}";

    # source file must exist, muquit, Sep-19-1999
    if (! -f "${sdir}/${file}")
    {
        print STDERR " *Error: ${sdir}/${file} does not exist\n";
        return;
    }

    # if the to file does not exist, copy it anyone
    # muquit, Sep-19-1999
    if (! -f "${tdir}/${file}")
    {
        $cmpout="do it anyone";
    }
    else
    {
        $cmpout = `$command`;
    }
    my($tofile) = $file;
    if($strippaths and ( $file =~ /\//)) 
    {
        $tofile =~ s/.*\///;
    }

    if($cmpout) 
    {
        $command="/bin/cp -r $sdir/$file $tdir/$tofile";
        print "Syncing $sdir/$file\n";
#        print "$command\n";
        $cpout = `$command`;
        $cpout and print STDERR "cp $file : $cpout\n";
    }
    else
    {
#        print STDERR "Not changed: $sdir/$file, $tdir/$tofile\n";
    }
}
